from manifest_edit.plugin.mpd import ManifestIteratorPlugin
from manifest_edit.context import Context
from manifest_edit import libfmp4
from schema import Schema, Or


class Plugin(ManifestIteratorPlugin):
    '''
    essential_property plugin.

    The purpose of this plugin is to add an essential property with the
    configured schemeIdUri and Value as a EssentialProperty, in the
    position specified in the configuration file.
    '''

    _name = __name__

    _keys = ['essentialPropertyScheme', 'essentialPropertyValue']

    # This is just the specific ["how" config] for this plugin
    def schema(self):
        return Schema(
            {
                self._keys[0]: str,
                self._keys[1]: Or(str, int)
            }
        )

    def addEssentialProperty(self, manifest, storage):
        for ess_property_config, element in self.config(manifest, storage):
            if not type(element) == libfmp4.mpd.Period:
                ess_property = libfmp4.mpd.Descriptor(
                    ess_property_config[self._keys[0]],
                    ess_property_config[self._keys[1]]
                )
                # Check if the property already exists. If it does, do
                #  nothing and log a warning
                if ess_property not in element.essentialProperties:
                    element.essentialProperties.append(ess_property)
                else:
                    Context.log_warning('The current manifest already has an '
                                        'essential property {}'.
                                        format(ess_property)
                                        )

            else:
                Context.log_warning('Cannot add Essential Property to a '
                                    'period, only adaptation sets or '
                                    'representations')

    def process(self, manifest, storage):
        self.addEssentialProperty(manifest, storage)
